@@ -15,6 +15,8 @@
 
 #include "tqslwiz.h"
 #include "wxutil.h"
+#include "tqsltrace.h"
+#include <wx/tokenzr.h>
 
 
 using namespace std;
@@ -35,6 +37,7 @@ END_EVENT_TABLE()
 
 void
 TQSLWizard::OnPageChanged(wxWizardEvent& ev) {
+	tqslTrace("TQSLWizard::OnPageChanged");
 	((TQSLWizPage *)GetCurrentPage())->SetFocus();
 	ExtWizard::OnPageChanged(ev);
 }
@@ -42,6 +45,7 @@ TQSLWizard::OnPageChanged(wxWizardEvent& ev) {
 TQSLWizard::TQSLWizard(tQSL_Location locp, wxWindow *parent, wxHtmlHelpController *help,
 	const wxString& title, bool expired) :
 	ExtWizard(parent, help, title), loc(locp), _curpage(-1) {
+	tqslTrace("TQSLWizard::TQSLWizard", "locp=0x%lx, parent=0x%lx, title=%s, expired=%d", (void *)locp, (void *)parent, _S(title), expired);
 
 	char buf[256];
 	if (!tqsl_getStationLocationCaptureName(locp, buf, sizeof buf)) {
@@ -54,6 +58,7 @@ TQSLWizard::TQSLWizard(tQSL_Location locp, wxWindow *parent, wxHtmlHelpControlle
 
 TQSLWizPage *
 TQSLWizard::GetPage(bool final) {
+	tqslTrace("TQSLWizard::GetPage", "final=%d", final);
 	int page_num;
 	page_changing = false;
 	if (final)
@@ -80,9 +85,9 @@ void TQSLWizCertPage::OnSize(wxSizeEvent& ev) {
 	UpdateFields();
 }
 
-
 TQSLWizPage *
 TQSLWizCertPage::GetPrev() const {
+	tqslTrace("TQSLWizCertPage::GetPrev");
 	int rval;
 	if (tqsl_hasPrevStationLocationCapture(loc, &rval) || !rval) {
 		GetParent()->page_changing = false;
@@ -97,6 +102,7 @@ TQSLWizCertPage::GetPrev() const {
 
 TQSLWizPage *
 TQSLWizCertPage::GetNext() const {
+	tqslTrace("TQSLWizCertPage::GetNext");
 	TQSLWizPage *newp;
 	bool final = false;
 	if (GetParent()->page_changing) {
@@ -112,6 +118,7 @@ TQSLWizCertPage::GetNext() const {
 
 void
 TQSLWizCertPage::UpdateFields(int noupdate_field) {
+	tqslTrace("TQSLWizCertPage::UpdateFields", "noupdate_field=%d", noupdate_field);
 	wxSize text_size = getTextSize(this);
 
 	if (noupdate_field >= 0)
@@ -153,9 +160,15 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 			char gabbi_name[40];
 			tqsl_getLocationFieldDataGABBI(loc, i, gabbi_name, sizeof gabbi_name);
 			int selected;
+			bool defaulted = false;
 			tqsl_getLocationFieldIndex(loc, i, &selected);
 			int new_sel = 0;
 			wxString old_sel = ((wxComboBox *)controls[i])->GetStringSelection();
+			if (old_sel.IsEmpty() && strcmp(gabbi_name, "CALL") == 0) { 
+				old_sel = ((TQSLWizard*)GetParent())->GetDefaultCallsign();
+				if (!old_sel.IsEmpty())
+					defaulted = true;		// Set from default
+			}
 			((wxComboBox *)controls[i])->Clear();
 			int nitems;
 			tqsl_getNumLocationFieldListItems(loc, i, &nitems);
@@ -170,7 +183,7 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 					*p = '\0';
 				wxString item_text(item, wxConvLocal);
 				((wxComboBox *)controls[i])->Append(item_text);
-				if (item_text == old_sel)
+				if (item_text == old_sel) 
 					new_sel = j;
 				wxCoord w, h;
 				((wxComboBox *)controls[i])->GetTextExtent(item_text, &w, &h);
@@ -182,7 +195,7 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 				((wxComboBox *)controls[i])->GetSize(&w, &h);
 				((wxComboBox *)controls[i])->SetSize(text_width + text_size.GetWidth()*4, h);
 			}
-			if (noupdate_field < 0)
+			if (noupdate_field < 0 && !defaulted)
 				new_sel = selected;
 			((wxComboBox *)controls[i])->SetSelection(new_sel);
 			tqsl_setLocationFieldIndex(loc, i, new_sel);
@@ -203,7 +216,6 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 		} else if (in_type == TQSL_LOCATION_FIELD_BADZONE) {
 			int len;
 			tqsl_getLocationFieldDataLength(loc, i, &len);
-			len /= 2;
 			int w, h;
 			((wxStaticText *)controls[i])->GetSize(&w, &h);
 			((wxStaticText *)controls[i])->SetSize((len+1)*text_size.GetWidth(), h);
@@ -212,6 +224,8 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 			((wxStaticText *)controls[i])->SetLabel(wxString(buf, wxConvLocal));
 			if (strlen(buf) == 0) {
 				this->GetParent()->FindWindow(wxID_FORWARD)->Enable(true);
+				if (valMsg)
+					((wxStaticText *)controls[i])->SetLabel(wxString(valMsg, wxConvLocal));
 			} else {
 				this->GetParent()->FindWindow(wxID_FORWARD)->Enable(false);
 			}
@@ -221,6 +235,7 @@ TQSLWizCertPage::UpdateFields(int noupdate_field) {
 
 void
 TQSLWizCertPage::OnComboBoxEvent(wxCommandEvent& event) {
+	tqslTrace("TQSLWizCertPage::OnComboBoxEvent");
 	int control_idx = event.GetId() - TQSL_ID_LOW;
 	if (control_idx < 0 || control_idx >= (int)controls.size())
 		return;
@@ -242,9 +257,11 @@ TQSLWizCertPage::OnCheckBoxEvent(wxCommandEvent& event) {
 
 TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 	: TQSLWizPage(parent, locp) {
+	tqslTrace("TQSLWizCertPage::TQSLWizCertPage", "parent=0x%lx, locp=0x%lx", (void *)parent, (void *)locp);
 	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
 	int control_width = getTextSize(this).GetWidth() * 20;
 	
+	valMsg = NULL;
 	tqsl_getStationLocationCapturePage(loc, &loc_page);
 	wxScreenDC sdc;
 	int label_w = 0;
@@ -264,14 +281,15 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 	bool addCheckbox = false;
 	wxString cbLabel;
 	for (int i = 0; i < numf; i++) {
-		wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);
 		char label[256];
 		int in_type, flags;
+		wxBoxSizer *hsizer;
 		tqsl_getLocationFieldDataLabel(loc, i, label, sizeof label);
 		tqsl_getLocationFieldInputType(loc, i, &in_type);
 		if (in_type != TQSL_LOCATION_FIELD_BADZONE) {
+			hsizer = new wxBoxSizer(wxHORIZONTAL);
 			hsizer->Add(new wxStaticText(this, -1, wxString(label, wxConvLocal), wxDefaultPosition,
-				wxSize(label_w, -1), wxALIGN_RIGHT|wxST_NO_AUTORESIZE), 0, wxTOP, 5);
+				wxSize(label_w, -1), wxALIGN_RIGHT/*|wxST_NO_AUTORESIZE*/), 0, wxTOP, 5);
 		}
 		wxWindow *control_p = 0;
 		char gabbi_name[256];
@@ -301,8 +319,12 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 				break;
 		}
 		controls.push_back(control_p);
-		hsizer->Add(control_p, 0, wxLEFT|wxTOP, 5);
-		sizer->Add(hsizer, 0, wxLEFT|wxRIGHT, 10);
+		if (in_type != TQSL_LOCATION_FIELD_BADZONE) {
+			hsizer->Add(control_p, 0, wxLEFT|wxTOP, 5);
+			sizer->Add(hsizer, 0, wxLEFT|wxRIGHT, 10);
+		} else {
+			sizer->Add(control_p, 0, wxEXPAND | wxLEFT| wxRIGHT, 10);
+		}
 	}
 
 	if (addCheckbox) {
@@ -323,9 +345,92 @@ TQSLWizCertPage::TQSLWizCertPage(TQSLWizard *parent, tQSL_Location locp)
 TQSLWizCertPage::~TQSLWizCertPage() {
 }
 
+const char *
+TQSLWizCertPage::validate() {
+	tqslTrace("TQSLWizCertPage::validate");
+	static char errtxt[128];
+	wxString error;
+	error = wxT("");
+	tqsl_setStationLocationCapturePage(loc, loc_page);
+	for (int i = 0; i < (int)controls.size(); i++) {
+		char gabbi_name[40];
+		tqsl_getLocationFieldDataGABBI(loc, i, gabbi_name, sizeof gabbi_name);
+		if (strcmp(gabbi_name, "GRIDSQUARE") == 0) {
+			wxString gridVal = ((wxTextCtrl *)controls[i])->GetValue();
+			if (gridVal.size() == 0) {
+				return 0;
+			}
+			wxString editedGrids = wxT("");
+			wxStringTokenizer grids(gridVal, wxT(","));	// Comma-separated list of squares
+			while (grids.HasMoreTokens()) {
+				wxString grid = grids.GetNextToken().Trim().Trim(false);
+				// Truncate to six character field
+				grid = grid.Left(6);
+				if (grid[0] <= 'z' && grid[1] >= 'a')
+					grid[0] = grid[0] - 'a' + 'A';	// Upper case first two
+				if (grid.size() > 1 && (grid[1] <= 'z' && grid[1] >= 'a'))
+					grid[1] = grid[1] - 'a' + 'A';
+				if (grid[0] < 'A' || grid[0] > 'R') {
+					if (error.IsEmpty())
+						error = wxString::Format(wxT("%s: Invalid Grid Square Field"), grid.c_str());
+				}
+				if (grid.size() > 1 && (grid[1] < 'A' || grid[1] > 'R')) {
+					if (error.IsEmpty()) 
+						error = wxString::Format(wxT("%s: Invalid Grid Square Field"), grid.c_str());
+				}
+				if (grid.size() > 2 && (grid[2] < '0' || grid[2] > '9')) {
+					if (error.IsEmpty()) 
+						error = wxString::Format(wxT("%s: Invalid Grid Square"), grid.c_str());
+				}
+				if (grid.size() < 4) {
+					if (error.IsEmpty()) 
+						error = wxString::Format(wxT("%s: Invalid Grid Square"), grid.c_str());
+				}
+				if (grid[3] < '0' || grid[3] > '9') {
+					if (error.IsEmpty()) 
+						error = wxString::Format(wxT("%s: Invalid Grid Square"), grid.c_str());
+				}
+
+				if (grid.size() > 4 && (grid[4] <= 'Z' && grid[4] >= 'A'))
+					grid[4] = grid[4] - 'A' + 'a';	// Lower case subsquare
+				if (grid.size() > 5 && (grid[5] <= 'Z' && grid[5] >= 'A'))
+					grid[5] = grid[5] - 'A' + 'a';
+
+				if (grid.size() > 4 && (grid[4] < 'a' || grid[4] > 'x')) {
+					if (error.IsEmpty()) 
+						error = wxString::Format(wxT("%s: Invalid Subsquare"), grid.c_str());
+				}
+				if (grid.size() > 5 && (grid[5] < 'a' || grid[5] > 'x')) {
+					if (error.IsEmpty()) 
+						error = wxString::Format(wxT("%s: Invalid Subsquare"), grid.c_str());
+				}
+				if (grid.size() != 6 && grid.size() != 4) {
+					// Not long enough yet or too long.
+					if (error.IsEmpty()) 
+						error = wxString::Format(wxT("%s: Invalid Grid Square"), grid.c_str());
+				}
+				if (!editedGrids.IsEmpty()) 
+					editedGrids += wxT(",");
+				editedGrids += grid;
+				((wxTextCtrl *)controls[i])->SetValue(editedGrids);
+				tqsl_setLocationFieldCharData(loc, i, ((wxTextCtrl *)controls[i])->GetValue().mb_str());
+			}
+			if (error.IsEmpty()) return 0;
+			strncpy(errtxt, error.mb_str(), sizeof errtxt);
+			return errtxt;
+		}
+	}
+	return 0;
+}
+
 bool
 TQSLWizCertPage::TransferDataFromWindow() {
-	GetParent()->page_changing = true;
+	tqslTrace("TQSLWizCertPage::TransferDataFromWindow");
+	valMsg = validate();
+	if (!valMsg) 
+		GetParent()->page_changing = true;
+	else
+		GetParent()->page_changing = false;
 	tqsl_setStationLocationCapturePage(loc, loc_page);
 	for (int i = 0; i < (int)controls.size(); i++) {
 		int in_type;
@@ -337,6 +442,12 @@ TQSLWizCertPage::TransferDataFromWindow() {
 			case TQSL_LOCATION_FIELD_TEXT:
 				tqsl_setLocationFieldCharData(loc, i, ((wxTextCtrl *)controls[i])->GetValue().mb_str());
 				break;
+#if 0
+			case TQSL_LOCATION_FIELD_BADZONE:
+				if (vmsg) 
+					tqsl_setLocationFieldCharData(loc, i, vmsg);
+				break;
+#endif
 		}
 	}
 	return true;
@@ -350,6 +461,7 @@ END_EVENT_TABLE()
 
 void
 TQSLWizFinalPage::OnListbox(wxCommandEvent &) {
+	tqslTrace("TQSLWizFinalPage::OnListbox");
 	if (namelist->GetSelection() >= 0) {
 		const char *cp = (const char *)(namelist->GetClientData(namelist->GetSelection()));
 		if (cp)
@@ -359,6 +471,7 @@ TQSLWizFinalPage::OnListbox(wxCommandEvent &) {
 
 TQSLWizFinalPage::TQSLWizFinalPage(TQSLWizard *parent, tQSL_Location locp, TQSLWizPage *i_prev) :
 	TQSLWizPage(parent, locp), prev(i_prev) {
+	tqslTrace("TQSLWizFinalPage::TQSLWizFinalPage", "parent=0x%lx, locp=0x%lx, i_prev=0x%lx", (void *) parent, (void *)locp, (void *) i_prev);
 	wxSize text_size = getTextSize(this);
 	int control_width = text_size.GetWidth()*30;
 	
@@ -402,6 +515,7 @@ TQSLWizFinalPage::TQSLWizFinalPage(TQSLWizard *parent, tQSL_Location locp, TQSLW
 
 bool
 TQSLWizFinalPage::TransferDataFromWindow() {
+	tqslTrace("TQSLWizFinalPage::TransferDataFromWindow");
 	if (validate())	// Must be a "back"
 		return true;
 	wxString s = newname->GetValue();
@@ -411,9 +525,10 @@ TQSLWizFinalPage::TransferDataFromWindow() {
 
 const char *
 TQSLWizFinalPage::validate() {
+	tqslTrace("TQSLWizFinalPage::validate");
 	wxString val = newname->GetValue();
 	const char *errmsg = 0;
-	val.Trim();
+	val.Trim().Trim(false);
 	if (val == wxT(""))
 		errmsg = "Station name must be provided";
 	return errmsg;
