@@ -15,9 +15,13 @@
 #include "sysconfig.h"
 #endif
 
+class CertTreeItemData;
+class LocTreeItemData;
 void displayCertProperties(CertTreeItemData *item, wxWindow *parent = 0);
+void displayLocProperties(LocTreeItemData *item, wxWindow *parent = 0);
 int getPassword(char *buf, int bufsiz, void *);
 void displayTQSLError(const char *pre);
 wxMenu *makeCertificateMenu(bool enable, bool keyonly = false);
+wxMenu *makeLocationMenu(bool enable);
 
 #endif	// __util_h
