@@ -11,13 +11,12 @@ cp LICENSE.txt $WORKDIR/
 cp apps/quick "$WORKDIR/Quick Start.txt"
 mkdir $WORKDIR/TrustedQSL
 cp -r apps/tqsl.app $WORKDIR/TrustedQSL
-cp -r apps/tqslcert.app $WORKDIR/TrustedQSL
 
 /bin/echo "done"
 
 /bin/echo -n "Installing the libraries and tweaking the binaries to look for them... "
 
-for app in tqsl tqslcert
+for app in tqsl
 do
     cp $TQSLLIBPATH $WORKDIR/TrustedQSL/$app.app/Contents/MacOS
     install_name_tool -change $TQSLLIBPATH @executable_path/libtqsllib.dylib $WORKDIR/TrustedQSL/$app.app/Contents/MacOS/$app
@@ -29,7 +28,6 @@ done
 /bin/echo -n "Installing the help... "
 
 cp -r apps/help/tqslapp $WORKDIR/TrustedQSL/tqsl.app/Contents/Resources/Help
-cp -r apps/help/tqslcert $WORKDIR/TrustedQSL/tqslcert.app/Contents/Resources/Help
 
 /bin/echo "done"
 
